import * as types from '../../types';
import * as control from './control';
declare abstract class AbstractCommon {
    get filename(): string;
    get controlName(): string;
    set controlName(v: string);
    get taskId(): number;
    get formId(): number;
    set formFocus(b: boolean);
    get path(): string;
    get prep(): string;
    get locale(): string;
    get l(): (key: string, data?: string[]) => string;
    get classPrepend(): (cla: any) => string;
    watch<T extends this, TK extends keyof T, TR>(name: TK | (() => TR), cb: (val: T[TK] & TR, old: T[TK] & TR) => void | Promise<void>, opt?: {
        'immediate'?: boolean;
        'deep'?: boolean;
    }): () => void;
    get refs(): Record<string, HTMLElement & control.AbstractControl & Record<string, any>>;
    get nextTick(): () => Promise<void>;
    allowEvent(e: MouseEvent | TouchEvent | KeyboardEvent): boolean;
    trigger(name: types.TGlobalEvent, param1?: boolean | Error | string, param2?: string): void;
    send(fid: number, obj: Record<string, any>): void;
    onBeforeCreate(): void | Promise<void>;
    onCreated(): void | Promise<void>;
    onBeforeMount(): void | Promise<void>;
    onBeforeUpdate(): void | Promise<void>;
    onUpdated(): void | Promise<void>;
    onBeforeUnmount(): void | Promise<void>;
    onUnmounted(): void | Promise<void>;
}
export declare abstract class AbstractPanel extends AbstractCommon {
    get panelId(): number;
    get rootForm(): AbstractForm & Record<string, any>;
    get rootPanel(): control.AbstractControl & Record<string, any>;
    get formHash(): string;
    set formHash(fh: string);
    get formHashData(): Record<string, any>;
    set formHashData(v: Record<string, any>);
    formHashBack(): Promise<void>;
    sendToRootPanel(data: Record<string, any>): void;
    enterStep(list: Array<{
        'value': string;
        'label'?: string;
        'icon'?: string;
        'desc'?: string;
    }>): Promise<boolean>;
    doneStep(): Promise<void>;
    qs: Record<string, string>;
    clearQs(): void;
    get formFocus(): boolean;
    onShow(e: types.IAbstractPanelShowEvent): void | Promise<void>;
    onShowed(): void | Promise<void>;
    onHide(): void | Promise<void>;
    onMounted(): void | Promise<void>;
    onReceive(data: Record<string, any>): void | Promise<void>;
    onQsChange(): void | Promise<void>;
    onQsChangeShow(e: types.IAbstractPanelQsChangeShowEvent): void | Promise<void>;
}
export declare abstract class AbstractForm extends AbstractCommon {
    isNativeSync: boolean;
    isReady: boolean;
    get formHash(): string;
    set formHash(v: string);
    get formHashData(): Record<string, any>;
    set formHashData(v: Record<string, any>);
    get topMost(): boolean;
    set topMost(v: boolean);
    get bottomMost(): boolean;
    set bottomMost(v: boolean);
    get isMask(): boolean;
    get formFocus(): boolean;
    get showInSystemTask(): boolean;
    set showInSystemTask(v: boolean);
    ready(cb: () => void | Promise<void>): void;
    formHashBack(): Promise<void>;
    sendToPanel(panel: control.AbstractControl & Record<string, any>, data: Record<string, any>): void;
    private _loading;
    get loading(): boolean;
    set loading(val: boolean);
    lockLoading: boolean;
    private _inStep;
    get inStep(): boolean;
    private readonly _stepValues;
    enterStep(list: Array<{
        'value': string;
        'label'?: string;
        'icon'?: string;
        'desc'?: string;
    }>): Promise<boolean>;
    updateStep(index: number, value: string): boolean;
    doneStep(): Promise<void>;
    private _firstShow;
    show(): void;
    showDialog(): Promise<string>;
    hide(): void;
    close(): void;
    dialogResult: string;
    onMounted(data: Record<string, any>): void | Promise<void>;
    onReceive(data: Record<string, any>): void | Promise<void>;
    onScreenResize(): void | Promise<void>;
    onConfigChanged<T extends keyof types.IConfig>(n: keyof types.IConfig, v: types.IConfig[T]): void;
    onFormCreated(taskId: number, formId: number, title: string, icon: string, showInSystemTask: boolean): void | Promise<void>;
    onFormRemoved(taskId: number, formId: number, title: string, icon: string): void | Promise<void>;
    onFormTitleChanged(taskId: number, formId: number, title: string): void | Promise<void>;
    onFormIconChanged(taskId: number, formId: number, icon: string): void | Promise<void>;
    onFormStateMinChanged(taskId: number, formId: number, state: boolean): void | Promise<void>;
    onFormStateMaxChanged(taskId: number, formId: number, state: boolean): void | Promise<void>;
    onFormShowChanged(taskId: number, formId: number, state: boolean): void | Promise<void>;
    onFormFocused(taskId: number, formId: number): void | Promise<void>;
    onFormBlurred(taskId: number, formId: number): void | Promise<void>;
    onFormFlash(taskId: number, formId: number): void | Promise<void>;
    onFormShowInSystemTaskChange(taskId: number, formId: number, value: boolean): void | Promise<void>;
    onFormHashChange(taskId: number, formId: number, value: string, data: Record<string, any>): void | Promise<void>;
    onTaskStarted(taskId: number): void | Promise<void>;
    onTaskEnded(taskId: number): void | Promise<void>;
    onLauncherFolderNameChanged(id: string, name: string): void | Promise<void>;
    onHashChanged(hash: string): void | Promise<void>;
    onKeydown(e: KeyboardEvent): void | Promise<void>;
    onKeyup(e: KeyboardEvent): void | Promise<void>;
}
export declare let simpleSystemTaskRoot: types.IVue;
export declare let launcherRoot: types.IVue;
export declare const elements: {
    'wrap': HTMLDivElement;
    'list': HTMLDivElement;
    'popList': HTMLDivElement;
    'circular': HTMLDivElement;
    'rectangle': HTMLDivElement;
    'gesture': HTMLDivElement;
    'drag': HTMLDivElement;
    'notify': HTMLElement;
    'alert': HTMLElement;
    'simpletask': HTMLDivElement;
    'launcher': HTMLDivElement;
    'confirm': HTMLDivElement;
    'init': () => void;
};
export declare function superConfirm(html: string): Promise<boolean>;
export declare function min(formId: number): boolean;
export declare function max(formId: number): boolean;
export declare function close(formId: number): boolean;
export declare function bindResize(e: MouseEvent | TouchEvent, border: types.TDomBorder): void;
export declare function bindDrag(e: MouseEvent | TouchEvent): void;
export declare function refreshMaxPosition(): void;
export declare function getTaskId(formId: number): number;
export declare function get(formId: number): types.IFormInfo | null;
export declare function send(formId: number, obj: Record<string, any>): void;
export declare function getList(taskId: number): Record<string, types.IFormInfo>;
export declare function getFocus(): number | null;
export declare const activePanels: Record<string, number[]>;
export declare function getActivePanel(formId: number): number[];
export declare function removeActivePanel(panelId: number, formId: number, taskId?: number): boolean;
export declare function setActivePanel(panelId: number, formId: number, taskId?: number): boolean;
export declare function hash(hash: string, formId: number): boolean;
export declare function getHash(formId: number): string;
export declare function hashBack(formId: number): Promise<boolean>;
export declare function changeFocus(formId?: number | string): void;
export declare function getMaxZIndexID(out?: {
    'taskIds'?: number[];
    'formIds'?: number[];
}): number | null;
export declare function getRectByBorder(border: types.TDomBorderCustom): {
    'width': number;
    'height': number;
    'left': number;
    'top': number;
};
export declare function showCircular(x: number, y: number): void;
export declare function moveRectangle(border: types.TDomBorderCustom): void;
export declare function showRectangle(x: number, y: number, border: types.TDomBorderCustom): void;
export declare function hideRectangle(): void;
export declare function showDrag(): void;
export declare function moveDrag(opt: types.IMoveDragOptions): void;
export declare function hideDrag(): void;
export declare function alert(content: string, type?: 'default' | 'primary' | 'info' | 'warning' | 'danger'): number;
export declare function notify(opt: types.INotifyOptions): number;
export declare function notifyProgress(notifyId: number, per: number): void;
export declare function notifyContent(notifyId: number, opt: types.INotifyContentOptions): void;
export declare function hideNotify(notifyId: number): void;
export declare function appendToPop(el: HTMLElement): void;
export declare function removeFromPop(el: HTMLElement): void;
export declare function showPop(el: HTMLElement | types.IVue, pop: HTMLElement | types.IVue | undefined, direction: 'h' | 'v' | 't' | MouseEvent | TouchEvent | {
    x: number;
    y: number;
}, opt?: {
    'size'?: {
        width?: number;
        height?: number;
    };
    'null'?: boolean;
    'autoPosition'?: boolean;
    'autoScroll'?: boolean;
    'flow'?: boolean;
    'way'?: 'normal' | 'click' | 'hover';
}): void;
export declare function hidePop(pop?: HTMLElement | types.IVue): void;
export declare function isJustPop(el: HTMLElement | number): boolean;
export declare function doFocusAndPopEvent(e: MouseEvent | TouchEvent): void;
export declare function remove(formId: number): boolean;
export declare function removePanel(id: number, vapp: types.IVApp, el: HTMLElement): boolean;
export declare function createPanel<T extends AbstractPanel>(rootPanel: control.AbstractControl, cls: string | (new () => T), opt?: {
    'layout'?: string;
    'style'?: string;
    'path'?: string;
}, taskId?: number): Promise<{
    'id': number;
    'vapp': types.IVApp;
    'vroot': T;
}>;
export declare function create<T extends AbstractForm>(cls: string | (new () => T), data?: Record<string, any>, opt?: {
    'layout'?: string;
    'style'?: string;
    'path'?: string;
}, taskId?: number): Promise<T>;
export declare function dialog(opt: string | types.IFormDialogOptions): Promise<string>;
export declare function confirm(opt: string | types.IFormConfirmOptions): Promise<boolean | number>;
export declare function prompt(opt: string | types.IFormPromptOptions): Promise<string>;
export declare function flash(formId: number, taskId?: number): void;
export declare function showLauncher(): void;
export declare function hideLauncher(): void;
export {};
